`timescale 1ns / 1ps
//
// Copyright (C) 2005-2006 by egnite Software GmbH. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. Neither the name of the copyright holders nor the names of
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
// SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
// OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
// AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// For additional information see http://www.ethernut.de/
//

//
// $Log$
//

module REGISTER(NOE, NWE, NRESET, A, P26, D, reg0_RSCR, reg0_IMR,
reg1_IMR, reg0_irq, reg1_irq, reg_MMC, reg1_MMC, mmc_irq);

  input NRESET;       // Board reset, low active.

  // Register addresses
  parameter AD0_RSCR = 8'h00; // RS232 Command Register.
  parameter AD0_IMR = 8'h04;  // Interrupt Mask Register, low byte.
  parameter AD1_IMR = 8'h05;  // Interrupt Mask Register, hi byte.
  parameter AD0_IPR = 8'h0C;  // Signal Latch Register, low byte.
  parameter AD1_IPR = 8'h0D;  // Signal Latch Register, high byte.
  parameter AD0_MMC = 8'h14;  // MMC Data Register.
  parameter AD1_MMC = 8'h18;  // External Enable Register.
  parameter AD0_SER = 8'h1C;  // Version ID Register.

  // CPU interface.
  input [7:0]A;       // Address bus.
  inout [7:0]D;       // Data bus.
  input P26;          // Chip select 2, low active.
  input NWE;          // Write strobe, low active.
  input NOE;          // Read strobe, low active.

  input mmc_irq;


  output [7:0]reg0_RSCR;
  output [7:0]reg0_IMR;
  output [3:0]reg1_IMR;
  input [7:0]reg_MMC;
  output [2:0]reg1_MMC;

  input [7:0]reg0_irq;
  input [3:0]reg1_irq;

  reg [7:0]reg0_RSCR;
  reg [7:0]reg0_IMR;
  reg [3:0]reg1_IMR;
  reg [7:0]D_out;
  reg [2:0]reg1_MMC;

  assign NOECS = ~NOE & ~P26;
  always @(posedge NWE or negedge NRESET)
  begin
    if(NRESET == 0)
      reg0_RSCR <= 8'b00100000;
    else if((A == AD0_RSCR)&(P26 == 0))
      reg0_RSCR <= D;
  end

  always @(posedge NWE or negedge NRESET)
  begin
    if(NRESET == 0)
      reg0_IMR <= 8'b0;
    else if((A == AD0_IMR)&(P26 == 0))
      reg0_IMR <= D;
  end

  always @(posedge NWE or negedge NRESET)
  begin
    if(NRESET == 0)
      reg1_IMR <= 4'b0;
    else if((A == AD1_IMR)&(P26 == 0))
      reg1_IMR <= D[3:0];
  end

  always @(posedge NWE or negedge NRESET)
  begin
    if(NRESET == 0)
      reg1_MMC <= 3'b111;
    else if((A == AD1_MMC)&(P26 == 0))
      reg1_MMC <= D;
  end

  // Read address decode.
  always @(negedge NOE)
  begin
    D_out[7:0] = 0;
    if((A == AD0_RSCR) & (P26 == 0))
      D_out = reg0_RSCR;
    if((A == AD0_IMR) & (P26 == 0))
      D_out = reg0_IMR;
    if((A == AD1_IMR) & (P26 == 0))
      D_out = reg1_IMR;
    if((A == AD0_MMC) & (P26 == 0))
      D_out = reg_MMC;
    if((A == AD1_MMC) & (P26 == 0))
      D_out = reg1_MMC;
    if((A == AD0_IPR) & (P26 == 0))
      D_out = {mmc_irq,reg0_irq[6:0]};
    if((A == AD1_IPR) & (P26 == 0))
      D_out = reg1_irq;
    if((A == AD0_SER) & (P26 == 0))
      D_out = 8'h02;
  end

  assign D = NOECS ? D_out : 8'bzzzzzzzz;

endmodule

